% Interpolation par la méthode des différences divisées de Newton

clc; 
clear;
close all;

% Définition des fonctions f1 et f2
f1 = @(x) cos(x);
f2 = @(x) 1 ./ (1 + x.^2);

% Bornes de l'intervalle
a = -5;
b = 5;

% Choix du nombre de points pour l'interpolation
n_values = [3, 5, 11];

% Évaluation et tracé pour chaque valeur de n
x_fine = linspace(a, b, 1000); % Points pour affichage des courbes réelles
figure;
for idx = 1:length(n_values)
    n = n_values(idx);
    x_n = linspace(a, b, n); % Points d'interpolation
    y_f1 = f1(x_n);
    y_f2 = f2(x_n);
    
    % Calcul des coefficients de Newton
    coeffs_f1 = newton_coeffs(x_n, y_f1)% donne les coeffs du pol inter. de la 1ere ligne de la méthode de nexton
    coeffs_f2 = newton_coeffs(x_n, y_f2)
    
    % Évaluation des polynômes interpolants
    y_interp_f1 = newton_eval(coeffs_f1, x_n, x_fine);
    y_interp_f2 = newton_eval(coeffs_f2, x_n, x_fine);
    
    % Tracé des résultats
    subplot(2, length(n_values), idx);
    plot(x_fine, f1(x_fine), 'b-', 'LineWidth', 1.5); hold on;
    plot(x_fine, y_interp_f1, 'r--', 'LineWidth', 1.5);
    plot(x_n, y_f1, 'ko', 'MarkerFaceColor', 'k');
    title(['Interpolation de cos(x), n=' num2str(n)]);
    legend('cos(x)', 'Interp.', 'Points', 'Location', 'Best');
    grid on;
    
    subplot(2, length(n_values), idx + length(n_values));
    plot(x_fine, f2(x_fine), 'b-', 'LineWidth', 1.5); hold on;
    plot(x_fine, y_interp_f2, 'r--', 'LineWidth', 1.5);
    plot(x_n, y_f2, 'ko', 'MarkerFaceColor', 'k');
    title(['Interpolation de 1/(1+x^2), n=' num2str(n)]);
    legend('1/(1+x^2)', 'Interp.', 'Points', 'Location', 'Best');
    grid on;
end

% Fonction pour calculer les coefficients des différences divisées de
% Newton (algo de newton)
function coeffs = newton_coeffs(x, y)
    n = length(x);
    coeffs = y;
    for j = 2:n
        for i = n:-1:j
            coeffs(i) = (coeffs(i) - coeffs(i-1)) / (x(i) - x(i-j+1));
        end
    end
end

% Fonction pour évaluer le polynôme interpolant de Newton (algo de newton)
function y_interp = newton_eval(coeffs, x_nodes, x_eval)
    n = length(x_nodes);
    y_interp = coeffs(n) * ones(size(x_eval));
    for k = n-1:-1:1
        y_interp = y_interp .* (x_eval - x_nodes(k)) + coeffs(k);
    end
end